@@ -1072,7 +1072,7 @@ impl<'module> Generator<'module> {
     }
 
     fn pattern_assignments_doc(assignments: Vec<Assignment<'_>>) -> Document<'_> {
-        let assignments = assignments.into_iter().map(pattern::Assignment::into_doc);
+        let assignments = assignments.into_iter().map(Assignment::into_doc);
         join(assignments, line())
     }
 
@@ -1230,7 +1230,7 @@ pub(crate) fn guard_constant_expression<'a>(
             .iter()
             .find(|assignment| assignment.name == name)
             .map(|assignment| assignment.subject.clone().append(assignment.path.clone()))
-            .unwrap_or_else(|| name.to_doc())),
+            .unwrap_or_else(|| maybe_escape_identifier_doc(name))),
 
         expression => constant_expression(tracker, expression),
     }
@@ -1286,10 +1286,17 @@ pub(crate) fn constant_expression<'a>(
 
         Constant::Var { name, module, .. } => Ok({
             match module {
-                None => name.to_doc(),
-                Some(module) => docvec!["$", module, ".", name],
+                None => maybe_escape_identifier_doc(name),
+                Some(module) => {
+                    // JS keywords can be accessed here, but we must escape anyway
+                    // as we escape when exporting such names in the first place,
+                    // and the imported name has to match the exported name.
+                    docvec!["$", module, ".", maybe_escape_identifier_doc(name)]
+                }
             }
         }),
+
+        Constant::Invalid { .. } => panic!("invalid constants should not reach code generation"),
     }
 }
 
