@@ -98,11 +98,12 @@ impl ModuleDecoder {
         };
         Ok(TypeConstructor {
             publicity: self.publicity(reader.get_publicity()?),
-            origin: Default::default(),
+            origin: self.src_span(&reader.get_origin()?)?,
             module: reader.get_module()?.into(),
             parameters: read_vec!(reader.get_parameters()?, self, type_),
             typ: type_,
             deprecation,
+            documentation: self.optional_string(reader.get_documentation()?),
         })
     }
 
