@@ -2,7 +2,11 @@ use crate::{
     ast::TodoKind,
     diagnostic::{self, Diagnostic, Location},
     error::wrap,
-    type_::{self, pretty::Printer},
+    type_::{
+        self,
+        error::{LiteralCollectionKind, PanicPosition, TodoOrPanic},
+        pretty::Printer,
+    },
 };
 use camino::Utf8PathBuf;
 use debug_ignore::DebugIgnore;
@@ -128,7 +132,7 @@ impl TypeWarningEmitter {
         }
     }
 
-    pub fn emit(&self, warning: crate::type_::Warning) {
+    pub fn emit(&self, warning: type_::Warning) {
         self.emitter.emit(Warning::Type {
             path: self.module_path.clone(),
             src: self.module_src.clone(),
@@ -142,7 +146,7 @@ pub enum Warning {
     Type {
         path: Utf8PathBuf,
         src: EcoString,
-        warning: crate::type_::Warning,
+        warning: type_::Warning,
     },
     InvalidSource {
         path: Utf8PathBuf,
@@ -192,7 +196,7 @@ expression.",
                     if !typ.is_variable() {
                         text.push_str(&format!(
                             "\n\nHint: I think its type is `{}`.\n",
-                            type_::pretty::Printer::new().pretty_print(typ, 0)
+                            Printer::new().pretty_print(typ, 0)
                         ));
                     }
 
@@ -600,19 +604,56 @@ the same values.\n"
                     }
                 }
 
-                type_::Warning::CaseMatchOnLiteralTuple { location } => Diagnostic {
-                    title: "Redundant tuple".into(),
-                    text: "Case expressions can take multiple subjects directly.".into(),
-                    hint: Some(
-                        "You can pass the contents of the tuple directly, separated by commas."
-                            .into(),
+                type_::Warning::CaseMatchOnLiteralCollection { kind, location } => {
+                    let kind = match kind {
+                        LiteralCollectionKind::List => "list",
+                        LiteralCollectionKind::Tuple => "tuple",
+                        LiteralCollectionKind::Record => "record",
+                    };
+
+                    let title = format!("Redundant {kind}");
+                    let text = wrap(&format!(
+                        "Instead of building a {kind} and matching on it, \
+you can match on its contents directly.
+A case expression can take multiple subjects separated by commas like this:
+
+    case one_subject, another_subject {{
+      _, _ -> todo
+    }}
+
+See: https://tour.gleam.run/flow-control/multiple-subjects/"
+                    ));
+
+                    Diagnostic {
+                        title,
+                        text,
+                        hint: None,
+                        level: diagnostic::Level::Warning,
+                        location: Some(Location {
+                            src: src.clone(),
+                            path: path.to_path_buf(),
+                            label: diagnostic::Label {
+                                text: Some(format!("You can remove this {kind} wrapper")),
+                                span: *location,
+                            },
+                            extra_labels: Vec::new(),
+                        }),
+                    }
+                }
+
+                type_::Warning::CaseMatchOnLiteralValue { location } => Diagnostic {
+                    title: "Match on a literal value".into(),
+                    text: wrap(
+                        "Matching on a literal value is redundant since you \
+can already tell which branch is going to match with this value.",
                     ),
+                    hint: None,
                     level: diagnostic::Level::Warning,
                     location: Some(Location {
                         src: src.clone(),
                         path: path.to_path_buf(),
                         label: diagnostic::Label {
-                            text: Some("You can remove this tuple wrapper".into()),
+                            text: Some("There's no need to pattern match on this value".into()),
                             span: *location,
                         },
                         extra_labels: Vec::new(),
@@ -684,6 +725,111 @@ hidden from the package's documentation.",
                         }),
                     }
                 }
+                type_::Warning::RedundantAssertAssignment { location } => Diagnostic {
+                    title: "Redundant assertion".into(),
+                    text: "This assertion is redundant since the pattern covers all possibilities."
+                        .into(),
+                    hint: None,
+                    level: diagnostic::Level::Warning,
+                    location: Some(Location {
+                        label: diagnostic::Label {
+                            text: Some("You can remove this".into()),
+                            span: *location,
+                        },
+                        path: path.clone(),
+                        src: src.clone(),
+                        extra_labels: vec![],
+                    }),
+                },
+
+                type_::Warning::TodoOrPanicUsedAsFunction {
+                    kind,
+                    location,
+                    args_location,
+                    args,
+                } => {
+                    let title = match kind {
+                        TodoOrPanic::Todo => "Todo used as a function".into(),
+                        TodoOrPanic::Panic => "Panic used as a function".into(),
+                    };
+                    let label_location = match args_location {
+                        None => location,
+                        Some(location) => location,
+                    };
+                    let name = match kind {
+                        TodoOrPanic::Todo => "todo",
+                        TodoOrPanic::Panic => "panic",
+                    };
+                    let mut text = format!("`{name}` is not a function");
+                    match args {
+                        0 => text.push_str(&format!(
+                            ", you can just write `{name}` instead of `{name}()`."
+                        )),
+                        1 => text.push_str(
+                            " and will crash before it can do anything with this argument.",
+                        ),
+                        _ => text.push_str(
+                            " and will crash before it can do anything with these arguments.",
+                        ),
+                    };
+
+                    match args {
+                        0 => {}
+                        _ => text.push_str(&format!(
+                            "\n\nHint: if you want to display an error message you should write
+`{name} as \"my error message\"`
+See: https://tour.gleam.run/advanced-features/{name}/"
+                        )),
+                    }
+
+                    Diagnostic {
+                        title,
+                        text: wrap(&text),
+                        hint: None,
+                        level: diagnostic::Level::Warning,
+                        location: Some(Location {
+                            label: diagnostic::Label {
+                                text: None,
+                                span: *label_location,
+                            },
+                            path: path.clone(),
+                            src: src.clone(),
+                            extra_labels: vec![],
+                        }),
+                    }
+                }
+
+                type_::Warning::UnreachableCodeAfterPanic {
+                    location,
+                    panic_position: unreachable_code_kind,
+                } => {
+                    let text = match unreachable_code_kind {
+                        PanicPosition::PreviousExpression =>
+                            "This code is unreachable because it comes after a `panic`.",
+                        PanicPosition::PreviousFunctionArgument =>
+                            "This argument is unreachable because the previous one always panics. \
+Your code will crash before reaching this point.",
+                        PanicPosition::LastFunctionArgument =>
+                            "This function call is unreachable because its last argument always panics. \
+Your code will crash before reaching this point.",
+                    };
+
+                    Diagnostic {
+                        title: "Unreachable code".into(),
+                        text: wrap(text),
+                        hint: None,
+                        level: diagnostic::Level::Warning,
+                        location: Some(Location {
+                            label: diagnostic::Label {
+                                text: None,
+                                span: *location,
+                            },
+                            path: path.clone(),
+                            src: src.clone(),
+                            extra_labels: vec![],
+                        }),
+                    }
+                }
             },
         }
     }
